﻿------------------------------------------------------------
-- RaidDebuff.lua
--
-- Abin
-- 2010/10/16
------------------------------------------------------------

local pairs = pairs
local ipairs = ipairs
local wipe = wipe
local GetSpellInfo = GetSpellInfo
local strtrim = strtrim
local tonumber = tonumber
local format = format
local StaticPopup_Show = StaticPopup_Show
local StaticPopup_Hide = StaticPopup_Hide
local GameTooltip = GameTooltip

local L = CompactRaid:GetLocale("RaidDebuff")
local module = CompactRaid:GetModule("RaidDebuff")
if not module then return end

local templates = CompactRaid.optionTemplates
local isUserInstance, selectedInstance, isUserBoss, selectedBoss, selectedDebuff, selectedSpellId

local GUID = "{104E8F0F-BB58-4F71-84F9-ED64D3163B18}"
local popupFunc, editboxText
local popupData = {
	button1 = OKAY,
	button2 = CANCEL,
	timeout = 0,
	exclusive = 1,
	whileDead = 1,
	hideOnEscape = 1,

	OnAccept = function(self)
		return popupFunc and popupFunc(self.editBox and self.editBox:GetText())
	end,

	EditBoxOnEnterPressed = function(self)
		local parent = self:GetParent()
		if not popupFunc or not popupFunc(self:GetText()) then
			parent:Hide()
		end
	end,

	EditBoxOnEscapePressed = function(self)
		self:GetParent():Hide()
	end,

	OnShow = function(self)
		if editboxText then
			self.editBox:SetText(editboxText)
			self.editBox:SetFocus()
			self.editBox:HighlightText()
		end
	end,
}

StaticPopupDialogs[GUID] = popupData

--------------------------------------------------
-- Option Page
--------------------------------------------------

local page = module.optionPage
page.sampleX, page.sampleY = 172, -256

function page:OnSampleShow()
	module.sampleDebuff:Show()
end

function page:OnSampleHide()
	module.sampleDebuff:Hide()
end

local scaleoffset = templates:CreateScaleOffsetGroup(page)
page.scaleoffset = scaleoffset
page:AnchorToTopLeft(scaleoffset)

function scaleoffset:OnScaleApply(scale)
	module.db.scale = scale
	scale = scale / 100
	module:ApplyDebuffScale(CompactRaid.sampleOverlay, scale)
	CompactRaid:EnumFrames(module, "ApplyDebuffScale", scale)
end

function scaleoffset:OnScaleCancel()
	return module.db.scale
end

function scaleoffset:OnOffsetApply(xoffset, yoffset)
	module.db.xoffset, module.db.yoffset = xoffset, yoffset
	module:ApplyDebuffOffset(CompactRaid.sampleOverlay, xoffset, yoffset)
	CompactRaid:EnumFrames(module, "ApplyDebuffOffset", xoffset, yoffset)
end

function scaleoffset:OnOffsetCancel()
	return module.db.xoffset, module.db.yoffset
end

local instanceCombo = page:CreateComboBox(INSTANCE)
page.instanceCombo = instanceCombo
instanceCombo:SetPoint("TOPLEFT", scaleoffset, "BOTTOMLEFT", 0, -26)
instanceCombo:SetWidth(186)

local bossCombo = page:CreateComboBox(BOSS)
page.bossCombo = bossCombo
bossCombo:SetPoint("LEFT", instanceCombo, "RIGHT", 4, 0)
bossCombo:SetWidth(instanceCombo:GetWidth())

local debuffList = UICreateVirtualScrollList(page:GetName().."DebuffList", page, 10, 1)
debuffList:SetHeight(202)
debuffList:SetPoint("TOPLEFT", instanceCombo, "BOTTOMLEFT", 4, -28)
debuffList:SetPoint("TOPRIGHT", bossCombo, "BOTTOMRIGHT", -4, -28)

local spellList = UICreateVirtualScrollList(page:GetName().."SpellList", page, 10, 1)
spellList:SetAllPoints(debuffList)
spellList:Hide()

local panel = page:CreatePanel()
panel:SetPoint("TOPLEFT", debuffList, "TOPLEFT", -4, 5)
panel:SetPoint("BOTTOMRIGHT", debuffList,"BOTTOMRIGHT",  4, -4)

local listLabel = panel:CreateFontString(nil, "ARTWORK", "GameFontNormalLeft")
listLabel:SetPoint("BOTTOMLEFT", panel, "TOPLEFT")
listLabel:SetText(L["debuff list"])

local listLabel2 = debuffList:CreateFontString(nil, "ARTWORK", "GameFontNormalRight")
listLabel2:SetPoint("BOTTOMRIGHT", panel, "TOPRIGHT", -20, 0)
listLabel2:SetText(L["priority"])

local newButton = page:CreatePressButton(NEW)
newButton:SetParent(debuffList)
newButton:SetPoint("TOPRIGHT", panel, "BOTTOM", -2, -6)
newButton:Disable()

local delButton = page:CreatePressButton(DELETE)
delButton:SetParent(debuffList)
delButton:SetPoint("LEFT", newButton, "RIGHT", 4, 0)
delButton:Disable()

local okayButton = page:CreatePressButton(OKAY)
okayButton:SetParent(spellList)
okayButton:SetAllPoints(newButton)
okayButton:Disable()

local cancelButton = page:CreatePressButton(CANCEL)
cancelButton:SetParent(spellList)
cancelButton:SetAllPoints(delButton)

local function GetUserStatusName(object)
	local name = object.name
	local status = object:GetUserStatus()
	if status == 2 then
		return name.." **"
	elseif status == 1 then
		return name.." *"
	else
		return name
	end
end

local function RefreshCombos()
	local instance = module:GetInstance(selectedInstance)
	if instance then
		instanceCombo.dropdown.text:SetText(GetUserStatusName(instance))
		local boss = instance:GetBoss(selectedBoss)
		if boss then
			bossCombo.dropdown.text:SetText(GetUserStatusName(boss))
		end
	end
end

newButton:SetScript("OnClick", function(self)
	popupData.text = L["new debuff"]
	popupData.hasEditBox = 1
	editboxText = nil
	popupFunc = function(name)
		name = strtrim(name or "")
		if name == "" then
			return 1
		end

		local instance = module:GetInstance(selectedInstance)
		if not instance then
			return
		end

		local id, spell, rank, icon
		id = tonumber(name)

		spellList:Clear()
		if id then
			spell, rank, icon = GetSpellInfo(id)
			local data = instance.debuffs[spell]
			if data then
				module:Print(format(L["debuff already exists"], spell, data.boss), 1, 0, 0)
				return 1
			end

			if spell then
				spellList:InsertData({ id = id, name = spell, rank = rank, icon = icon, link = "|cff71d5ff|Hspell:"..id.."|h["..spell.."]|h|r" })
			end
		else
			local data = instance.debuffs[name]
			if data then
				module:Print(format(L["debuff already exists"], name, data.boss), 1, 0, 0)
				return 1
			end

			for id = 1, 100000 do
				spell, rank, icon = GetSpellInfo(id)
				if spell and spell == name then
					spellList:InsertData({ id = id, name = spell, rank = rank, icon = icon, link = "|cff71d5ff|Hspell:"..id.."|h["..spell.."]|h|r" })
				end
			end
		end

		if spellList:GetDataCount() > 0 then
			spellList:Show()
			spellList:SetSelection(1)
		else
			module:Print(format(L["debuff not exists"], name), 1, 0, 0)
		end
	end

	StaticPopup_Show(GUID)
end)

delButton:SetScript("OnClick", function(self)
	local instance = module:GetInstance(selectedInstance)
	if instance then
		local boss = instance:GetBoss(selectedBoss)
		if boss and boss:UnregisterDebuff(selectedDebuff) then
			debuffList:RemoveData(debuffList:GetSelection())
			RefreshCombos()
		end
	end
end)

local function PriorityCombo_OnChange(self, value)
	local data = self.debuffData
	if not data then
		return
	end

	data.level = value
	if data.defLevel == value then
		module.chardb[data.id] = nil
	else
		module.chardb[data.id] = value
	end
end

function debuffList:OnButtonCreated(button)
	button.icon = button:CreateTexture(nil, "ARTWORK")
	button.icon:SetWidth(16)
	button.icon:SetHeight(16)
	button.icon:SetPoint("LEFT", 4, 0)
	button.icon:SetTexCoord(0.08, 0.92, 0.08, 0.92)

	button.priority = page:CreateComboBox()
	button.priority:SetParent(button)
	button.priority:SetWidth(100)
	button.priority:SetPoint("RIGHT", -2, 0)
	button.priority:SetBackdrop(nil)
	button.priority:EnableMouse(false)
	button.priority.toggleButton:SetScale(0.8)
	button.priority.dropdown.text:SetJustifyH("RIGHT")
	button.priority:AddLine(L["critical"], module.DEBUFF_LEVEL_HIGH, nil, nil, 1, 0, 0)
	button.priority:AddLine(L["normal"], module.DEBUFF_LEVEL_NORMAL)
	button.priority:AddLine(L["low"], module.DEBUFF_LEVEL_LOW, nil, nil, 0.5, 0.5, 1)
	button.priority:AddLine(L["ignore"], module.DEBUFF_LEVEL_NONE, nil, nil, 0.5, 0.5, 0.5)
	button.priority.OnComboChanged = PriorityCombo_OnChange

	button.text = button:CreateFontString(nil, "ARTWORK", "GameFontHighlightLeft")
	button.text:SetPoint("LEFT", button.icon, "RIGHT", 4, 0)
	button.text:SetPoint("RIGHT", button.priority, "LEFT")
end

function debuffList:OnButtonUpdate(button, data)
	button.icon:SetTexture(data.icon)
	button.text:SetText(data.name..(data.user and " **" or ""))
	button.priority.debuffData = data
	button.priority:SetSelection(data.level, 1)
end

function debuffList:OnButtonTooltip(button, data)
	GameTooltip:SetHyperlink(data.link)
end

function debuffList:OnButtonClick(button, data)
	module.sampleDebuff.icon:SetTexture(button.icon:GetTexture())
	HandleModifiedItemClick(data.link)
end

function debuffList:OnSelectionChanged(position, data)
	if data and data.user then
		selectedDebuff = data.id
		delButton:Enable()
	else
		selectedDebuff = nil
		delButton:Disable()
	end
end

local function RefreshDebuffList()
	debuffList:Clear()
	local instance = module:GetInstance(selectedInstance)
	local boss = instance and instance:GetBoss(selectedBoss)
	if boss then
		local data
		for _, data in pairs(boss.debuffs) do
			if data.boss == selectedBoss then
				debuffList:InsertData(data)
			end
		end
	end
end

page:HookScript("OnShow", RefreshDebuffList)

debuffList:HookScript("OnHide", function(self)
	StaticPopup_Hide(GUID)
end)

spellList:HookScript("OnShow", function(self)
	StaticPopup_Hide(GUID)
	debuffList:Hide()
	listLabel:SetText(format(L["select spell"], OKAY))
end)

spellList:HookScript("OnHide", function(self)
	StaticPopup_Hide(GUID)
	self:Hide()
	debuffList:Show()
	listLabel:SetText(L["debuff list"])
end)

function spellList:OnButtonCreated(button)
	button.icon = button:CreateTexture(nil, "ARTWORK")
	button.icon:SetWidth(16)
	button.icon:SetHeight(16)
	button.icon:SetPoint("LEFT", 4, 0)
	button.icon:SetTexCoord(0.08, 0.92, 0.08, 0.92)

	button.text = button:CreateFontString(nil, "ARTWORK", "GameFontHighlightLeft")
	button.text:SetPoint("LEFT", button.icon, "RIGHT", 4, 0)
end

function spellList:OnButtonUpdate(button, data)
	button.icon:SetTexture(data.icon)
	if data.rank then
		button.text:SetText(data.name.." |cff808080"..data.rank.."|r")
	else
		button.text:SetText(data.name)
	end
end

function spellList:OnButtonTooltip(button, data)
	GameTooltip:SetHyperlink(data.link)
end

function spellList:OnButtonClick(button, data)
	HandleModifiedItemClick(data.link)
end

function spellList:OnSelectionChanged(position, data)
	if data then
		selectedSpellId = data.id
		okayButton:Enable()
	else
		selectedSpellId = nil
		okayButton:Disable()
	end
end

okayButton:SetScript("OnClick", function(self)
	spellList:Hide()
	local instance = module:GetInstance(selectedInstance)
	local boss = instance and instance:GetBoss(selectedBoss)
	if boss and selectedSpellId then
		local data = boss:RegisterDebuff(selectedSpellId, module.DEBUFF_LEVEL_NORMAL, 1)
		if data then
			local position = debuffList:InsertData(data)
			debuffList:SetSelection(position)
			debuffList:EnsureVisible()
			RefreshCombos()
		end
	end
end)

cancelButton:SetScript("OnClick", function(self)
	spellList:Hide()
end)

function instanceCombo:OnMenuRequest()
	local i, instance
	for i, instance in ipairs(module.instanceList) do
		self:AddLine(GetUserStatusName(instance), instance.name)
	end

	if selectedInstance and isUserInstance then
		self:AddLine(DELETE.."...", "delete", nil, 4, 1, 0, 0)
	end
	self:AddLine(NEW.."...", "new", nil, 4, 0, 1, 0)
end

function instanceCombo:OnComboChanged(value)
	spellList:Hide()
	StaticPopup_Hide(GUID)

	if value == "delete" and selectedInstance then
		popupData.text = format(L["delete instance"], selectedInstance)
		popupData.hasEditBox = nil
		editboxText = nil
		popupFunc = function()
			module:UnregisterInstance(selectedInstance)
			instanceCombo:SetSelection()
			module:ZONE_CHANGED_NEW_AREA()
		end

		StaticPopup_Show(GUID)

	elseif value == "new" then
		popupData.text = L["new instance"]
		popupData.hasEditBox = 1
		editboxText = nil
		local zone = GetRealZoneText()
		if not module:GetInstance(zone) then
			editboxText = zone
		end

		popupFunc = function(name)
			name = strtrim(name or "")
			if name == "" or name == "delete" or name == "new" then
				return 1
			end

			if module:GetInstance(name) then
				module:Print(format(L["instance already exists"], name), 1, 0, 0)
				return 1
			end

			local instance = module:RegisterInstance(name, 1)
			if not instance then
				return 1
			end

			instanceCombo:AddLine(name, name)
			instanceCombo:SetSelection(name)
			module:ZONE_CHANGED_NEW_AREA()
		end

		StaticPopup_Show(GUID)
	else
		selectedInstance, isUserInstance = nil
		bossCombo:ClearLines()

		local instance = module:GetInstance(value)
		if instance then
			selectedInstance = value
			isUserInstance = instance.user
			bossCombo:OnMenuRequest()
			if #instance.bosses > 0 then
				bossCombo:SetSelectionByPosition(1)
			end
		end
	end
end

function bossCombo:OnMenuRequest()
	local instance = module:GetInstance(selectedInstance)
	if instance then
		local i, boss
		for i, boss in ipairs(instance.bosses) do
			self:AddLine(GetUserStatusName(boss), boss.name)
		end

		if selectedBoss and isUserBoss then
			self:AddLine(DELETE.."...", "delete", nil, 4, 1, 0, 0)
		end

		self:AddLine(NEW.."...", "new", nil, 4, 0, 1, 0)
	end
end

function bossCombo:OnComboChanged(value)
	spellList:Hide()
	StaticPopup_Hide(GUID)

	if value == "delete" and selectedInstance and selectedBoss then
		popupData.text = format(L["delete boss"], selectedBoss)
		popupData.hasEditBox = nil
		popupFunc = function()
			local instance = module:GetInstance(selectedInstance)
			if instance and instance:UnregisterBoss(selectedBoss) then
				RefreshCombos()
				bossCombo:SetSelection()
			end
		end

		StaticPopup_Show(GUID)

	elseif value == "new" then
		popupData.text = L["new boss"]
		popupData.hasEditBox = 1
		editboxText = nil
		if GetRealZoneText() == selectedInstance then
			local target = UnitExists("target") and not UnitIsPlayer("target") and not UnitPlayerControlled("target") and UnitName("target") or nil
			local instance = module:GetInstance(selectedInstance)
			if instance and not instance:GetBoss(target) then
				editboxText = target
			end
		end

		popupFunc = function(name)
			name = strtrim(name or "")
			if name == "delete" or name == "new" then
				return 1
			end

			if name == "" then
				name = module.COMMON_MOBS
			end

			local instance = module:GetInstance(selectedInstance)
			if not instance then
				return 1
			end

			if instance:GetBoss(name) then
				module:Print(format(L["boss already exists"], name), 1, 0, 0)
				return 1
			end

			local boss = instance:RegisterBoss(name, 1)
			if not boss then
				return 1
			end

			bossCombo:AddLine(name, name)
			bossCombo:SetSelection(name)
			RefreshCombos()
		end

		StaticPopup_Show(GUID)
	else
		selectedBoss, isUserBoss = nil
		newButton:Disable()
		delButton:Disable()
		local instance = module:GetInstance(selectedInstance)
		local boss = instance and instance:GetBoss(value)
		if boss then
			selectedBoss = value
			isUserBoss = boss.user
			newButton:Enable()
		end

		RefreshDebuffList()
	end
end