------------------------------------------------------------
-- OptionTemplates.lua
--
-- Abin
-- 2010/10/31
------------------------------------------------------------

local type = type
local tostring = tostring
local tonumber = tonumber
local pairs = pairs
local CreateFrame = CreateFrame
local StaticPopup_Hide = StaticPopup_Hide
local GameTooltip = GameTooltip
local NORMAL_FONT_COLOR = NORMAL_FONT_COLOR
local ColorPickerFrame = ColorPickerFrame

local _, addon = ...
local L = addon.L
local templates = {}
addon.optionTemplates = templates

-- Just do a "function" type check before calling, but I do need the error messages to popup!
local function _pcall(func, ...)
	if type(func) == "function" then
		return func(...)
	end
end

local pageCount = 0
function templates:CreateOptionFrame(name, title, version, desc, dualTalents, parent)
	if type(name) ~= "string" then
		pageCount = pageCount + 1
		name = "CompactRaidOptionPage_UnnamedPage"..pageCount
	end

	local frame = UICreateInterfaceOptionPage(name, title, desc, parent)
	frame.version = frame:CreateFontString(name.."Version", "ARTWORK", "GameFontNormalSmall")
	frame.version:SetPoint("LEFT", frame.title, "RIGHT", 8, 0)
	frame.version:SetText(version)

	if dualTalents then
		local dualTalentText = frame:CreateFontString(frame:GetName().."DualTalentText", "ARTWORK", "GameFontNormalSmall")
		frame.dualTalentText = dualTalentText
		dualTalentText:SetPoint("LEFT", frame.version, "RIGHT", 8, 0)
	end

	frame:HookScript("OnShow", function(self)
		if self.OnSampleShow then
			local sample = addon.sampleButtonFrame
			sample:SetParent(self)
			sample:Show()
			sample:SetFrameStrata("FULLSCREEN_DIALOG")

			local position = addon.db.samplePositions[self.moduleName]
			local x = position and position.x or self.sampleX
			local y = position and position.y or self.sampleY

			if type(x) == "number" and type(y) == "number" then
				sample:ClearAllPoints()
				sample:SetPoint("TOPLEFT", x, y)
			elseif not sample:GetLeft() then
				sample:ClearAllPoints()
				sample:SetPoint("TOPLEFT", 120, -70)
			end

			self:OnSampleShow(addon.sampleOverlay)
		end
	end)

	frame:HookScript("OnHide", function(self)
		StaticPopup_Hide(addon.guid)
		addon.sampleButtonFrame:Hide()
		if self.OnSampleHide then
			self:OnSampleHide(addon.sampleOverlay)
		end
	end)

	return frame
end

function templates:CreateScrollFrame(name, parent, scrollRespondingFrame, createChild)
	local frame = CreateFrame("ScrollFrame", name, parent, "UIPanelScrollFrameTemplate")
	frame.scrollBarHideable = 1
	frame:SetSize(300, 310)

	local scrollBarBk = CreateFrame("Frame", nil, frame)
	scrollBarBk:SetBackdrop({ edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", edgeSize = 16, insets = {left = 5, right = 5, top = 5, bottom = 5 }})
	scrollBarBk:SetBackdropBorderColor(0.75, 0.75, 0.75, 0.75)
	scrollBarBk:SetPoint("TOPLEFT", _G[name.."ScrollBarScrollUpButton"], "TOPLEFT", -4, 4)
	scrollBarBk:SetPoint("BOTTOMRIGHT", _G[name.."ScrollBarScrollDownButton"], "BOTTOMRIGHT", 2, -4)
	scrollBarBk:SetParent(_G[name.."ScrollBar"])
	scrollBarBk:SetScale(0.6)

	if createChild then
		local child = CreateFrame("Frame", name.."ScrollChild", frame)
		child:SetSize(10, 10)
		frame:SetScrollChild(child)
	end

	local respondFrame = scrollRespondingFrame or parent or frame
	respondFrame:EnableMouseWheel(true)
	respondFrame:SetScript("OnMouseWheel", function(self, delta) ScrollFrameTemplate_OnMouseWheel(frame, delta) end)

	frame:HookScript("OnScrollRangeChanged", function(self, xOffset, yOffset)
		if yOffset == 0 then
			_G[self:GetName().."ScrollBar"]:Hide()
		else
			_G[self:GetName().."ScrollBar"]:Show()
		end
	end)
	return frame
end

local editToolltip = CreateFrame("Button", "CompactRaidOptionPageTooltip", UIParent)
templates.editToolltip = editToolltip

editToolltip:SetBackdrop({ edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", edgeSize = 16, insets = {left = 5, right = 5, top = 5, bottom = 5 }, })
--editToolltip:SetBackdropColor(TOOLTIP_DEFAULT_BACKGROUND_COLOR.r, TOOLTIP_DEFAULT_BACKGROUND_COLOR.g, TOOLTIP_DEFAULT_BACKGROUND_COLOR.b)
editToolltip:SetFrameStrata("TOOLTIP")
editToolltip:SetClampedToScreen(true)
editToolltip:SetWidth(240)

local tooltipBk = editToolltip:CreateTexture(nil, "BACKGROUND")
tooltipBk:SetPoint("TOPLEFT", 4, -4)
tooltipBk:SetPoint("BOTTOMRIGHT", -4, 4)
tooltipBk:SetTexture(0, 0, 0, 1)

editToolltip.text = editToolltip:CreateFontString(editToolltip:GetName().."Text", "ARTWORK", "GameFontHighlightSmallLeftTop")
editToolltip.text:SetPoint("TOPLEFT", 10, -10)
editToolltip.text:SetWidth(220)
editToolltip.text:SetTextColor(1, 0.5, 0)
editToolltip.text:SetNonSpaceWrap(true)
editToolltip:Hide()

function editToolltip:Display(editbox)
	self.text:SetText(editbox:IsMultiLine() and L["edit prompt multiline"] or L["edit prompt"])
	self:SetHeight(self.text:GetHeight() + 20)
	self:ClearAllPoints()
	self:SetPoint("TOPLEFT", editbox, "BOTTOMLEFT")
	self:Show()
end

function templates:SetEditBoxAutoPrompt(editbox)

	editbox:HookScript("OnEditFocusGained", function(self)
		editToolltip.owner = self
		editToolltip:Display(self)
	end)

	editbox:HookScript("OnEditFocusLost", function(self)
		if editToolltip.owner == self then
			editToolltip.owner = nil
			editToolltip:Hide()
		end
	end)
end

function templates:CreateSubEdit(parent, frame, text, key, disableInCombat, textColor)
	local edit = parent:CreateEditBox(text, true, disableInCombat, textColor)
	edit.key = key
	frame.editboxes[key] = edit
	edit:SetParent(frame)
	edit:SetWidth(50)
	edit:SetJustifyH("CENTER")
	edit.default = 0

	edit.text:ClearAllPoints()
	edit:SetPoint("LEFT", edit.text, "RIGHT", 2, 0)

	templates:SetEditBoxAutoPrompt(edit)

	edit.ValidateValue = function(self, value)
		if type(value) ~= "number" then
			return self.default
		elseif self.min and value < self.min then
			return self.min
		elseif self.max and value > self.max then
			return self.max
		end
	end

	edit.OnTextValidate = function(self, text)
		local err = self:ValidateValue(tonumber(text))
		if err then
			return 1, tostring(err)
		end
	end

	edit.OnTextCommit = function(self, text)
		self:GetParent():OnTextCommit(self, self:GetNumber())
	end

	edit.OnTextCancel = function(self)
		local value = self:GetParent():OnTextCancel(self)
		if value then
			return tostring(value)
		end
	end

	edit.Commit = function(self)
		local text = self:GetText()
		local invalid, newText = self:OnTextValidate(text)
		if invalid then
			text = newText
			self:SetText(text)
		end

		self:ClearFocus()
		return tonumber(text)
	end

	return edit
end

function templates:CreateScaleOffsetGroup(parent, disableInCombat, textColor)
	local frame = parent:CreateSubControl("Frame")
	frame:SetSize(26, 26)
	frame.editboxes = {}

	local scale = templates:CreateSubEdit(parent, frame, L["scale"], "scale", disableInCombat, textColor)
	scale.min, scale.max, scale.default = 10, 300, 100
	scale:SetNumeric(true)
	scale.text:ClearAllPoints()
	scale.text:SetPoint("LEFT", frame, "LEFT")
	scale:SetPoint("LEFT", scale.text, "RIGHT", 2, 0)

	local percLabel = scale:CreateFontString(nil, "ARTWORK", "GameFontNormalLeft")
	scale.percLabel = percLabel
	percLabel:SetText("%")
	percLabel:SetPoint("LEFT", scale, "RIGHT")

	local xoffset = templates:CreateSubEdit(parent, frame, L["x-offset"], "xoffset", disableInCombat, textColor)
	xoffset.text:SetPoint("LEFT", percLabel, "RIGHT", 8, 0)

	local yoffset =  templates:CreateSubEdit(parent, frame, L["y-offset"], "yoffset", disableInCombat, textColor)
	yoffset.text:SetPoint("LEFT", xoffset, "RIGHT", 8, 0)

	frame.OnTextCommit = function(self, editbox, value)
		if editbox.key == "scale" then
			_pcall(self.OnScaleApply, self, value)
		elseif editbox.key == "xoffset" or editbox.key == "yoffset" then
			if editbox.key == "xoffset" then
				_pcall(self.OnOffsetApply, self, value, self.editboxes.yoffset:Commit())
			elseif editbox.key == "yoffset" then
				_pcall(self.OnOffsetApply, self, self.editboxes.xoffset:Commit(), value)
			end
		end
	end

	frame.OnTextCancel = function(self, editbox)
		if editbox.key == "scale" then
			return _pcall(self.OnScaleCancel, self)
		elseif editbox.key == "xoffset" or editbox.key == "yoffset" then
			local x, y = _pcall(self.OnOffsetCancel, self)
			return editbox.key == "xoffset" and x or y
		end
	end

	frame.ClearFocus = function(self)
		local editbox
		for _, editbox in pairs(self.editboxes) do
			editbox:ClearFocus()
		end
	end

	frame.SetFocus = function(self, key)
		local editbox = self.editboxes[key]
		if editbox then
			editbox:SetFocus()
		end
	end

	frame.SetValue = function(self, key, value, value2)
		if key == "offset" then
			if type(value) ~= "number" then
				value = 0
			end
			self.editboxes.xoffset:SetNumber(value)

			if type(value2) ~= "number" then
				value2 = 0
			end
			self.editboxes.yoffset:SetNumber(value2)
			return value, value2
		else
			local edit = self.editboxes[key]
			if edit then
				local err = edit:ValidateValue(value)
				if err then
					value = err
				end
				edit:SetNumber(value)
				return value
			end
		end
	end

	return frame
end

-- Make the stupid ColorPickerFrame callback system a little bit object-oriented
local function SwatchFunc(restore)
	local swatch = templates.selectedColorSwatch
	if not swatch or not swatch[addon.guid] then
		return
	end

	local r, g, b
	if restore then
		r, g, b = restore.r, restore.g, restore.b
	else
		r, g, b = ColorPickerFrame:GetColorRGB()
	end

	swatch:SetColor(r, g, b)
	_pcall(swatch.OnColorChange, swatch, r, g, b)
	if restore then
		_pcall(swatch.OnColorCancel, swatch, r, g, b)
	end
end

function templates:CreateColorSwatch(name, parent)
	local colorSwatch = CreateFrame("Button", name, parent)
	colorSwatch[addon.guid] = 1
	colorSwatch:SetWidth(16)
	colorSwatch:SetHeight(16)
	colorSwatch.r, colorSwatch.g, colorSwatch.b = 0, 1, 0

	colorSwatch:SetBackdrop({ bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = true, tileSize = 8 })
	colorSwatch:SetBackdropColor(0, 0, 0, 1)

	colorSwatch.bg = colorSwatch:CreateTexture(nil, "BORDER")
	colorSwatch.bg:SetPoint("TOPLEFT", 1, -1)
	colorSwatch.bg:SetPoint("BOTTOMRIGHT", -1, 1)
	colorSwatch.bg:SetTexture(HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b)

	colorSwatch.swatch = colorSwatch:CreateTexture(nil, "ARTWORK")
	colorSwatch.swatch:SetPoint("TOPLEFT", 2, -2)
	colorSwatch.swatch:SetPoint("BOTTOMRIGHT", -2, 2)
	colorSwatch.swatch:SetTexture(0, 1, 0)

	colorSwatch._Enable = colorSwatch.Enable
	colorSwatch.Enable = function(self)
		self.swatch:SetTexture(self.r, self.g, self.b)
		self:_Enable()
		_pcall(self.OnEnable, self)
	end

	colorSwatch._Disable = colorSwatch.Disable
	colorSwatch.Disable = function(self)
		if templates.selectedColorSwatch == self and ColorPickerFrame.func == SwatchFunc then
			templates.selectedColorSwatch = nil
			ColorPickerFrame:Hide()
		end
		self.swatch:SetTexture(0.25, 0.25, 0.25)
		self:_Disable()
		_pcall(self.OnDisable, self)
	end

	colorSwatch.SetColor = function(self, r, g, b)
		self.r, self.g, self.b = r, g, b
		if self:IsEnabled() then
			self.swatch:SetTexture(r, g, b)
		end
	end

	colorSwatch.GetColor = function(self)
		return self.r, self.g, self.b
	end

	colorSwatch:SetScript("OnEnter", function(self)
		self.bg:SetTexture(NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b)
		GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
		GameTooltip:ClearLines()
		GameTooltip:AddLine(L["click to change color"], nil, nil, nil, 1)
		GameTooltip:Show()
	end)

	colorSwatch:SetScript("OnLeave", function(self)
		GameTooltip:Hide()
		self.bg:SetTexture(HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b)
	end)

	colorSwatch:SetScript("OnClick", function(self)
		GameTooltip:Hide()
		ColorPickerFrame:Hide()
		templates.selectedColorSwatch = self
		ColorPickerFrame.func = SwatchFunc
		ColorPickerFrame:SetColorRGB(self.r, self.g, self.b)
		ColorPickerFrame.previousValues = { r = self.r, g = self.g, b = self.b }
		ColorPickerFrame.cancelFunc = SwatchFunc
		ColorPickerFrame.hasOpacity = nil
		ColorPickerFrame:Show()
	end)

	colorSwatch:SetScript("OnHide", function(self)
		if templates.selectedColorSwatch == self and ColorPickerFrame.func == SwatchFunc then
			templates.selectedColorSwatch = nil
			ColorPickerFrame:Hide()
		end
	end)

	return colorSwatch
end

function templates:CreateClassCustomColorGroup(parent, text, hasVehicle)
	local group = parent:CreateSingleSelectionGroup(text, 1)
	group:AddButton(L["class color"], 1)
	local customeButton = group:AddButton(CUSTOM, 0)

	local colorSwatch = templates:CreateColorSwatch(customeButton:GetName().."ColorSwatch", customeButton)
	group.colorSwatch = colorSwatch
	colorSwatch:SetPoint("LEFT", customeButton.text, "RIGHT", 4, 0)

	colorSwatch.OnColorChange = function(self, r, g, b)
		_pcall(group.OnColorChange, group, r, g, b)
		_pcall(group.OnApplyOption, group)
	end

	colorSwatch:SetScript("OnShow", function(self)
		local r, g, b = _pcall(group.OnColorRequest, group)
		if not r then
			r, g, b = 0, 1, 0
		end
		self:SetColor(r, g, b)
	end)

	group.OnCheckInit = function(self, value, vehicle)
		local checked = _pcall(self.OnOptionInit, self, value)
		if value == 1 then
			if checked then
				self.colorSwatch:Disable()
			else
				self.colorSwatch:Enable()
			end
		end
		return checked
	end

	group.OnSelectionChanged = function(self, value)
		if value == 1 then
			self.colorSwatch:Disable()
		else
			self.colorSwatch:Enable()
		end
		_pcall(self.OnOptionChange, self, value)
		_pcall(group.OnApplyOption, group)
	end

	if not hasVehicle then
		return group
	end

	local vehicleGroup = parent:CreateMultiSelectionGroup()
	local vehicleButton = vehicleGroup:AddButton(L["in vehicle"], 1)
	vehicleButton:ClearAllPoints()
	vehicleButton:SetPoint("LEFT", group.colorSwatch, "RIGHT", 16, 0)

	local vehicleSwatch = templates:CreateColorSwatch(vehicleButton:GetName().."ColorSwatch", vehicleButton)
	vehicleSwatch:SetPoint("LEFT", vehicleButton.text, "RIGHT", 4, 0)

	vehicleSwatch.OnColorChange = function(self, r, g, b)
		_pcall(group.OnColorChange, group, r, g, b, 1)
		_pcall(group.OnApplyOption, group, 1)
	end

	vehicleSwatch:SetScript("OnShow", function(self)
		local r, g, b = _pcall(group.OnColorRequest, group, 1)
		if not r then
			r, g, b = 0, 1, 0
		end
		self:SetColor(r, g, b)
	end)

	vehicleGroup.OnCheckInit = function(self, value, checked)
		local checked = _pcall(group.OnOptionInit, group, 1, 1)
		if checked then
			vehicleSwatch:Enable()
		else
			vehicleSwatch:Disable()
		end
		return checked
	end

	vehicleGroup.OnCheckChanged = function(self, value, checked)
		if checked then
			vehicleSwatch:Enable()
		else
			vehicleSwatch:Disable()
		end
		_pcall(group.OnOptionChange, group, checked, 1)
		_pcall(group.OnApplyOption, group, 1)
	end

	return group
end

local STOLEN_FLAG = "{04153BF0-0F4D-4C79-ACBB-596FE2139346}"

function templates:StealBlizzardOptionFrame(frame, newParent, initFunc, stolenFunc)
	if not frame then
		return
	end

	local data = frame[STOLEN_FLAG]
	if not data then
		data = {}
		frame[STOLEN_FLAG] = data
		if initFunc then
			initFunc(frame, data)
		end
	elseif data.stolen then
		return
	end

	data.stolen = 1
	data.parent = frame:GetParent()
	data.width = frame:GetWidth()
	data.height = frame:GetHeight()
	data.scale = frame:GetScale()
	data.numPoints = frame:GetNumPoints()
	local i
	for i = 1, data.numPoints do
		data["point"..i], data["relativeTo"..i], data["relativePoint"..i], data["xOffset"..i], data["yOffset"..i] = frame:GetPoint(i)
	end

	if newParent then
		frame:SetParent(newParent)
	end

	frame:ClearAllPoints()

	if stolenFunc then
		stolenFunc(frame, data)
	end

	return frame, data
end

function templates:ReturnBlizzardOptionFrame(frame, returnedFunc)
	if not frame then
		return
	end

	local data = frame[STOLEN_FLAG]
	if not data or not data.stolen then
		return
	end

	frame:SetParent(data.parent)
	if data.width > 0 and data.height > 0 then
		frame:SetSize(data.width, data.height)
	end
	frame:SetScale(data.scale)
	frame:ClearAllPoints()
	local i
	for i = 1, data.numPoints do
		frame:SetPoint(data["point"..i], data["relativeTo"..i], data["relativePoint"..i], data["xOffset"..i], data["yOffset"..i])
	end

	if returnedFunc then
		returnedFunc(frame, data)
	end

	data.stolen = nil
	return frame, data
end

local function SolenSlider_OnValueChanged(self, value)
	local data = self[STOLEN_FLAG]
	if data then
		data.value:SetText(value)
	end
end

local function SolenSlider_OnMinMaxChanged(self, minVal, maxVal)
	local data = self[STOLEN_FLAG]
	if data then
		data.low:SetText(minVal)
		data.high:SetText(maxVal)
	end
end

local function StolenSlider_OnInitialize(self, data)
	data.text = self:CreateFontString(nil, "ARTWORK", "GameFontHighlight")
	data.text:SetText(text)
	data.text:SetPoint("BOTTOMLEFT", self, "TOPLEFT")

	data.value = self:CreateFontString(nil, "ARTWORK", "GameFontGreen")
	data.value:SetPoint("BOTTOMRIGHT", self, "TOPRIGHT")

	data.low = self:CreateFontString(nil, "ARTWORK", "GameFontHighlightSmall")
	data.low:SetPoint("TOPLEFT", self, "BOTTOMLEFT", 0, 2)

	data.high = self:CreateFontString(nil, "ARTWORK", "GameFontHighlightSmall")
	data.high:SetPoint("TOPRIGHT", self, "BOTTOMRIGHT", 0, 2)

	data.value:SetText(self:GetValue())
	local minVal, maxVal = self:GetMinMaxValues()
	data.low:SetText(minVal)
	data.high:SetText(maxVal)

	self:HookScript("OnValueChanged", SolenSlider_OnValueChanged)
	self:HookScript("OnMinMaxChanged", SolenSlider_OnMinMaxChanged)
end

local function StolenSlider_OnStolen(self, data)
	data.text:Show()
	data.value:Show()
	data.low:Show()
	data.high:Show()
	_G[self:GetName().."Text"]:Hide()
	_G[self:GetName().."Low"]:Hide()
	_G[self:GetName().."High"]:Hide()
	self:SetSize(180, 16)
	self:SetScale(0.9)
end

local function StolenSlider_OnReturned(self, data)
	data.text:Hide()
	data.value:Hide()
	data.low:Hide()
	data.high:Hide()
	_G[self:GetName().."Text"]:Show()
	_G[self:GetName().."Low"]:Show()
	_G[self:GetName().."High"]:Show()
end

function templates:StealBlizzardSlider(slider, newParent)
	return self:StealBlizzardOptionFrame(slider, newParent, StolenSlider_OnInitialize, StolenSlider_OnStolen)
end

function templates:ReturnBlizzardSlider(slider)
	return self:ReturnBlizzardOptionFrame(slider, StolenSlider_OnReturned)
end

function templates:DisableBlizzardOption(button, value)
	if value then
		SetCVar(button.cvar, value)
	end
	button:Disable()
	_G[button:GetName().."Text"]:SetTextColor(GRAY_FONT_COLOR.r, GRAY_FONT_COLOR.g, GRAY_FONT_COLOR.b)
end