------------------------------------------------------------
-- SinglePartyFrame.lua
--
-- Abin
-- 2011/7/16
------------------------------------------------------------

local CreateFrame = CreateFrame
local RegisterStateDriver = RegisterStateDriver
local GetCVar = GetCVar
local RegisterUnitWatch = RegisterUnitWatch

local _, addon = ...
local layout = addon.layoutManager
local manager = addon.thirdPartyManager

local frame = CreateFrame("Frame", "CompactRaidExtendedPartyFrame", UIParent, "SecureFrameTemplate")
addon.partyFrame = frame
frame:SetSize(72, 199)
frame:SetPoint("LEFT", 100, 0)
frame:SetMovable(true)
frame:SetUserPlaced(true)
RegisterStateDriver(frame, "visibility", "[group:raid] hide; show")

function frame:UpdateVisibility()
	local state
	if GetCVar("useCompactPartyFrames") == "1" then
		if addon.db.single then
			state = "[group:raid] hide; show"
		else
			state = "[group:party] show; hide"
		end
	else
		if addon.db.single then
			state = "[nogroup] show; hide"
		else
			state = "hide"
		end
	end
	addon:CombatSafeCall(RegisterStateDriver, self, "visibility", state)
end

frame:RegisterEvent("CVAR_UPDATE")
frame:SetScript("OnEvent", function(self, event, cvar, value)
	if event == "CVAR_UPDATE" and cvar == "USE_RAID_STYLE_PARTY_FRAMES" then
		self:UpdateVisibility()
	end
end)

local petParent = CreateFrame("Frame", frame:GetName().."PetParent", frame, "SecureFrameTemplate")
addon.partyFrame.petParent = petParent
petParent:Hide()
petParent:SetAllPoints(frame)

local function PetFrame_OnEvent(self, event, unit, ...)
	if event == "UNIT_PET" then
		-- unit of this frame is always "pet" but I only check "player" here, that's why I cannot pass it to manager:ProcessEvent
		if unit == self._parentUnit then
			manager:UpdateFrame(self)
		end
	else
		manager:ProcessEvent(self, event, unit, ...)
	end
end

local function RemoveNonPetEvents(self)
	-- Some events are meaningless for a pet, just unregister them to save a little of CPU cycles
	self:UnregisterEvent("UNIT_CONNECTION")
	self:UnregisterEvent("PLAYER_ROLES_ASSIGNED")
	self:UnregisterEvent("UNIT_ENTERED_VEHICLE")
	self:UnregisterEvent("UNIT_EXITED_VEHICLE")
	self:UnregisterEvent("READY_CHECK")
	self:UnregisterEvent("READY_CHECK_FINISHED")
	self:UnregisterEvent("READY_CHECK_CONFIRM")
	self:UnregisterEvent("PARTY_MEMBER_DISABLE")
	self:UnregisterEvent("PARTY_MEMBER_ENABLE")
end

local function CreatePetFrame(id)
	local party = _G["CompactPartyFrameMember"..id]
	if not party then
		return
	end

	local parentUnit = id == 1 and "player" or "party"..(id - 1)
	local unit = id == 1 and "pet" or "partypet"..(id - 1)

	local pet = manager:CreateFrame(party:GetName().."PetFrame", petParent, unit, "mini")
	pet._parentUnit = parentUnit
	pet:Hide()
	manager["partyPetFrame"..id] = pet
	pet:SetPoint("BOTTOM", party, "BOTTOM")
	RemoveNonPetEvents(pet)
	pet:SetScript("OnEvent", PetFrame_OnEvent)

	local nextParty = id < 5 and _G["CompactPartyFrameMember"..(id + 1)]
	if nextParty then
		nextParty:ClearAllPoints()
		nextParty:SetPoint("TOP", pet, "BOTTOM")
	end

	local header = CreateFrame("Frame", nil, pet, "SecureHandlerShowHideTemplate")
	header:SetFrameRef("partyFrame", party)

	header:SetAttribute("_onshow", [[
		local pet = self:GetParent()
		pet:ClearAllPoints()
		pet:SetPoint("TOP", self:GetFrameRef("partyFrame"), "BOTTOM")
	]])

	header:SetAttribute("_onhide", [[
		local pet = self:GetParent()
		pet:ClearAllPoints()
		pet:SetPoint("BOTTOM", self:GetFrameRef("partyFrame"), "BOTTOM")
	]])

	if id == 1 then
		RegisterStateDriver(pet, "visibility", "[pet, group:party] show; hide")
	else
		RegisterUnitWatch(pet)
	end

	return pet
end

local function InitializePartyFrame()
	local i
	for i = 1, 5 do
		local party = _G["CompactPartyFrameMember"..i]
		party:SetParent(frame)
		local pet = CreatePetFrame(i)
		layout.partyAdjustFrames[party] = (i * 2) - 1
		layout.partyAdjustFrames[pet] = i * 2
	end

	CompactPartyFrameMember1:ClearAllPoints()
	CompactPartyFrameMember1:SetPoint("TOP", 0, -14)
	CompactPartyFrame:Hide()
	RegisterStateDriver(CompactPartyFrame, "visibility", "hide")
end

-- Classes those may have permenant pets
local PET_CLASSES = { HUNTER = 1, WARLOCK = 1, MAGE = 1, DEATHKNIGHT = 1 }

------------------------------------------------------------
-- Pet-party1 displaying mode (added in 1.3)
------------------------------------------------------------

local function CreateMyPetFrame()
	local petFrame = manager:CreateFrame("CompactRaidMyPetFrame", frame, "pet", "normal")
	petFrame._parentUnit = "player"
	petFrame:SetAllPoints(CompactPartyFrameMember2)
	manager.myPetFrame = petFrame
	layout.partyAdjustFrames[petFrame] = 2
	RemoveNonPetEvents(petFrame)
	petFrame:SetScript("OnEvent", PetFrame_OnEvent)
	RegisterStateDriver(petFrame, "visibility", "[pet, nogroup] show; hide")
end

function frame:OnPartyFramesCreated()
	addon:CombatSafeCall(InitializePartyFrame)
	if PET_CLASSES[select(2, UnitClass("player"))] then
		addon:CombatSafeCall(CreateMyPetFrame)
	end
	layout:CreatePartyContainer()
end

local partyCreated = CompactPartyFrame
hooksecurefunc("CompactPartyFrame_Generate", function()
	if not partyCreated then
		partyCreated = CompactPartyFrame
		if partyCreated then
			frame:OnPartyFramesCreated()
		end
	end
end)