------------------------------------------------------------
-- SampleButton.lua
--
-- Abin
-- 2010/10/16
------------------------------------------------------------

local select = select
local UnitClass = UnitClass
local UnitPowerType = UnitPowerType
local random = random
local GameTooltip = GameTooltip
local RAID_CLASS_COLORS = RAID_CLASS_COLORS
local PowerBarColor = PowerBarColor

local _, addon = ...
local L = addon.L
local manager = addon.buttonManager
local layout = addon.layoutManager

local frame = CreateFrame("Frame", "CompactRaidSampleButtonFrame", addon.optionPage)
addon.sampleButtonFrame = frame
frame:SetSize(72, 36)
frame:SetMovable(true)
frame:SetUserPlaced(false)
frame:SetDontSavePosition(true)

function frame:SavePosition()
	local parent = self:GetParent()
	local x, y = self:GetLeft(), self:GetTop()
	local px, py = parent:GetLeft(), parent:GetTop()
	if x and y and px and py then
		local db = addon.db.samplePositions[parent.moduleName]
		if not db then
			db = {}
			addon.db.samplePositions[parent.moduleName] = db
		end
		db.x, db.y = x - px, y - py
		--print(db.x, db.y) -- debug
	end
end

local panel = layout:CreateLayoutContainer(frame, 1, frame)
addon.sampleButtonPanel = panel
panel:SetSize(frame:GetWidth(), frame:GetHeight())
panel:SetPoint("CENTER")
panel.border:SetFrameLevel(1)

function panel:OnDragStop(self)
	frame:SavePosition()
end

local button = CreateFrame("Button", "CompactRaidSampleButton", panel)
addon.sampleButton = button
button.unit, button.displayedUnit = "player", "player"
local data = {}
button.data = data
button:SetAllPoints(panel)

button.background = button:CreateTexture(nil, "BACKGROUND")
button.background:SetPoint("TOPLEFT")
button.background:SetPoint("BOTTOMRIGHT", -1, 1)
button.background:SetTexture(0, 0, 0, 1)

button.healthBar = CreateFrame("StatusBar", button:GetName().."HealthBar", button)
button.healthBar:SetPoint("TOPLEFT", 1, -1)
button.healthBar:SetMinMaxValues(0, 10)
button.healthBar:SetValue(7)
button.healthBar.background = button.healthBar:CreateTexture(nil, "BORDER")
button.healthBar.background:SetAllPoints(button.healthBar)
data.incomingHeal = button.healthBar:CreateTexture()

button.powerBar = CreateFrame("StatusBar", button:GetName().."PowerBar", button)
button.powerBar:SetPoint("TOPLEFT", button.healthBar, "BOTTOMLEFT")
button.powerBar:SetPoint("BOTTOMRIGHT", -2, 2)
button.powerBar:SetMinMaxValues(0, 10)
button.powerBar:SetValue(3)
button.powerBar.background = button.powerBar:CreateTexture(nil, "BORDER")
button.powerBar.background:SetAllPoints(button.powerBar)

manager.SetBothBarTexture(button, "Interface\\RaidFrame\\Raid-Bar-Hp-Fill")

button.name = button.healthBar:CreateFontString(button:GetName().."NameText", "OVERLAY", "GameFontHighlightSmall")
button.name:SetText(L["sample frame"])
data.name = button.name:GetText()
button.statusText = button.healthBar:CreateFontString(button:GetName().."StatusText", "OVERLAY", "GameFontHighlightSmall")

button.roleIcon = button.healthBar:CreateTexture(nil, "OVERLAY")
button.roleIcon:SetSize(1, 12)
button.roleIcon:SetPoint("TOPLEFT", button.healthBar, "TOPLEFT")
button.roleIcon:SetTexture("Interface\\TargetingFrame\\UI-RaidTargetingIcons")
button.roleIcon:Hide()

function button:UpdateRaidIcon()
	if addon.db.showRaidIcon then
		SetRaidTargetIconTexture(self.roleIcon, random(8))
		self.roleIcon:SetWidth(12)
		self.roleIcon:Show()
	else
		self.roleIcon:SetWidth(1)
		self.roleIcon:Hide()
	end
end

local function CreateAuraFrame(name, icon, point, x, y, debuffColor, size)
	local frame = CreateFrame("Frame", name, button.healthBar, debuffColor and "DebuffButtonTemplate" or "AuraButtonTemplate")
	frame:SetSize(size or 10, size or 10)
	frame:SetPoint(point, x, y)
	frame:SetFrameLevel(7)
	_G[frame:GetName().."Icon"]:SetTexture(icon)

	if debuffColor then
		local border = _G[frame:GetName().."Border"]
		border:SetPoint("TOPLEFT", -1, 1)
		border:SetPoint("BOTTOMRIGHT", 1, -1)
		border:SetVertexColor(debuffColor.r, debuffColor.g, debuffColor.b)
	end

	frame:UnregisterAllEvents()
	frame:SetScript("OnEnter", nil)
	frame:SetScript("OnLeave", nil)

	frame.UpdateVisible = function(self, hide)
		if hide then
			self:Hide()
		else
			self:Show()
		end
	end

	return frame
end

button.buff = CreateAuraFrame(button:GetName().."Buff", "Interface\\Icons\\Spell_Holy_MagicalSentry", "BOTTOMRIGHT", -1, 1)
button.debuff = CreateAuraFrame(button:GetName().."Debuff", "Interface\\Icons\\Ability_Hunter_SniperShot", "BOTTOMLEFT", 1, 1, DebuffTypeColor.Magic)
button.dispel = CreateAuraFrame(button:GetName().."DispelDebuff", "Interface\\RaidFrame\\Raid-Icon-DebuffMagic", "TOPRIGHT", -1, -1, nil, 12)

button.buffFrames = {}
tinsert(button.buffFrames, button.buff)
button.debuffFrames = {}
tinsert(button.debuffFrames, button.debuff)
button.dispelDebuffFrames = {}
tinsert(button.dispelDebuffFrames, button.dispel)

local overlay = CreateFrame("Frame", button:GetName().."_CompactRaidAddOnOverlay", button)
addon.sampleOverlay = overlay
overlay:SetPoint("TOPLEFT", 1, -1)
overlay:SetPoint("BOTTOMRIGHT", -2, 2)
overlay:SetFrameLevel(10)

-- Highlight texture
data.highlight = button.healthBar:CreateTexture(nil, "OVERLAY")
data.highlight:SetAllPoints(button)
data.highlight:SetTexture("Interface\\QuestFrame\\UI-QuestTitleHighlight")
data.highlight:SetBlendMode("ADD")
data.highlight:SetVertexColor(1, 1, 1, 0.4)
data.highlight:Hide()

button:RegisterForDrag("LeftButton")
button:SetScript("OnDragStart", function()
	frame:StartMoving()
end)

button:SetScript("OnDragStop", function()
	frame:StopMovingOrSizing()
	frame:SavePosition()
end)

button:SetScript("OnEnter", function(self)
	data.highlight:Show()
	GameTooltip:SetOwner(self, "ANCHOR_LEFT")
	GameTooltip:ClearLines()
	GameTooltip:AddLine(L["sample frame"], 1, 1, 1)
	GameTooltip:AddLine(L["sample desc"], nil, nil, nil, 1)
	GameTooltip:Show()
end)

button:SetScript("OnLeave", function(self)
	data.highlight:Hide()
	GameTooltip:Hide()
end)

button:SetScript("OnShow", function(self)
	manager.CallGroupFunc("UpdateHealthColor", self)
	local powerColor = PowerBarColor[UnitPowerType("player")]
	button.powerBar:SetStatusBarColor(powerColor.r, powerColor.g, powerColor.b)
	manager.CallGroupFunc("UpdatePowerColor", self)
end)

hooksecurefunc(manager, "OnButtonSizeChanged", function(x, y)
	panel:SetSize(x, y)
end)