------------------------------------------------------------
-- ThirdPartyManager.lua
--
-- Abin
-- 2011/1/31
------------------------------------------------------------

local type = type
local CreateFrame = CreateFrame
local ipairs = ipairs
local tremove = tremove
local UnitExists = UnitExists
local UnitIsUnit = UnitIsUnit

local _, addon = ...
local manager = {}
addon.thirdPartyManager = manager
manager.frames = {}

-- Calling the original "CompactUnitFrame_UpdateAll" is not secure since which calls "CompactUnitFrame_UpdateVisible", and which,
-- in turn, calls frame:Show() or frame:Hide() according to the unit existance. Blizzard addons can do that in combat, we cannot.
local function CompactUnitFrame_UpdateAll(self)
	if UnitExists(self.displayedUnit) then
		CompactUnitFrame_UpdateMaxHealth(self)
		CompactUnitFrame_UpdateHealth(self)
		CompactUnitFrame_UpdateHealthColor(self)
		CompactUnitFrame_UpdateMaxPower(self)
		CompactUnitFrame_UpdatePower(self)
		CompactUnitFrame_UpdatePowerColor(self)
		CompactUnitFrame_UpdateName(self)
		CompactUnitFrame_UpdateSelectionHighlight(self)
		CompactUnitFrame_UpdateAggroHighlight(self)
		--CompactUnitFrame_UpdateInRange(self) -- This one is over ridden by addon.buttonManager utility
		CompactUnitFrame_UpdateStatusText(self)
		CompactUnitFrame_UpdateHealPrediction(self)
		CompactUnitFrame_UpdateRoleIcon(self)
		CompactUnitFrame_UpdateReadyCheck(self)
		CompactUnitFrame_UpdateAuras(self)
	end

	addon.buttonManager.CallGroupFunc("SetUnit", self)
end

-- Mostly copied from contents of "CompactUnitFrame_OnEvent", with some little changes to avoid taints
local function DefaultEventProc(frame, event, unit, ...)
	if event == "PLAYER_ENTERING_WORLD" then
		CompactUnitFrame_UpdateAll(frame)
	elseif event == "PLAYER_TARGET_CHANGED" then
		CompactUnitFrame_UpdateSelectionHighlight(frame)
	elseif event == "PLAYER_REGEN_ENABLED" or event == "PLAYER_REGEN_DISABLED" then
		CompactUnitFrame_UpdateAuras(frame)
	elseif event == "PLAYER_ROLES_ASSIGNED" then
		CompactUnitFrame_UpdateRoleIcon(frame)
	elseif event == "READY_CHECK" or event == "READY_CHECK_FINISHED" then
		CompactUnitFrame_UpdateReadyCheck(frame)
	elseif event == "PARTY_MEMBER_DISABLE" or event == "PARTY_MEMBER_ENABLE" then
		CompactUnitFrame_UpdateMaxPower(frame)
		CompactUnitFrame_UpdatePower(frame)
		CompactUnitFrame_UpdatePowerColor(frame)
	elseif unit and (UnitIsUnit(unit, frame.displayedUnit) or UnitIsUnit(unit, frame.unit)) then
		if event == "UNIT_MAXHEALTH" then
			CompactUnitFrame_UpdateMaxHealth(frame)
			CompactUnitFrame_UpdateHealth(frame)
			CompactUnitFrame_UpdateHealPrediction(frame)
		elseif event == "UNIT_HEALTH" then
			CompactUnitFrame_UpdateHealth(frame)
			CompactUnitFrame_UpdateStatusText(frame)
			CompactUnitFrame_UpdateHealPrediction(frame)
		elseif event == "UNIT_MAXPOWER" then
			CompactUnitFrame_UpdateMaxPower(frame)
			CompactUnitFrame_UpdatePower(frame)
		elseif event == "UNIT_POWER" then
			CompactUnitFrame_UpdatePower(frame)
		elseif event == "UNIT_DISPLAYPOWER" or event == "UNIT_POWER_BAR_SHOW" or event == "UNIT_POWER_BAR_HIDE" then
			CompactUnitFrame_UpdateMaxPower(frame)
			CompactUnitFrame_UpdatePower(frame)
			CompactUnitFrame_UpdatePowerColor(frame)
		elseif event == "UNIT_NAME_UPDATE" then
			CompactUnitFrame_UpdateName(frame)
		elseif event == "UNIT_AURA" then
			CompactUnitFrame_UpdateAuras(frame)
		elseif event == "UNIT_THREAT_SITUATION_UPDATE" then
			CompactUnitFrame_UpdateAggroHighlight(frame)
		elseif event == "UNIT_CONNECTION" then
			CompactUnitFrame_UpdateHealthColor(frame)
			CompactUnitFrame_UpdatePowerColor(frame)
			CompactUnitFrame_UpdateStatusText(frame)
		elseif event == "UNIT_HEAL_PREDICTION" then
			CompactUnitFrame_UpdateHealPrediction(frame)
		elseif event == "UNIT_ENTERED_VEHICLE" or event == "UNIT_EXITED_VEHICLE" or event == "UNIT_PET" then
			CompactUnitFrame_UpdateAll(frame)
		elseif event == "READY_CHECK_CONFIRM" then
			CompactUnitFrame_UpdateReadyCheck(frame)
		end
	end
end

manager.DefaultEventProc = DefaultEventProc

-- Creates a third-party unit frame using "CompactUnitFrameTemplate"
function manager:CreateFrame(name, parent, unit, style)
	if type(name) ~= "string" then
		return
	end

	local frame = CreateFrame("Button", name, parent, "CompactUnitFrameTemplate")
	if not frame then
		return
	end

	-- Unit frame popup menus are not allowed by Blizzard for third party frames! Perl classic causes errors directly, x-perl replaced the menu
	-- with raid-target icons, smart guy... Others I dunno, but there's no way to get it work whatsoever.
	local id, popup
	for id, popup in ipairs(UnitPopupFrames) do
		if popup == frame.dropDown:GetName() then
			tremove(UnitPopupFrames, id)
			break
		end
	end

	frame.menu = nil
	frame:SetAttribute("type",  "target")
	frame:SetAttribute("*type1",  nil)
	frame:SetAttribute("*type2",  nil)

	local data = {}
	self.frames[frame] = data

	data.style = select(2, addon.buttonManager.RegisterFrame(frame, style))
	manager:SetUnit(frame, unit)
	CompactUnitFrame_SetUpFrame(frame, data.style == "mini" and DefaultCompactMiniFrameSetup or DefaultCompactUnitFrameSetup)
	addon.buttonManager.UpdateNonBlizzardFrameSize(frame, data)
	frame:HookScript("OnEnter", CompactUnitFrame_UpdateAll)
	frame:SetScript("OnEvent", nil)
	return frame
end

-- Modified from "CompactUnitFrame_SetUnit" to avoid taints, you should NEVER call "CompactUnitFrame_SetUnit" for third-party frames!
function manager:SetUnit(frame, unit)
	if unit ~= frame.unit then
		frame.unit = unit
		frame.displayedUnit = unit
		frame.inVehicle = false
		frame:SetAttribute("unit", unit)
		CompactUnitFrame_UpdateAll(frame)
	end
end

-- Update frame contents.
function manager:UpdateFrame(frame)
	CompactUnitFrame_UpdateAll(frame)
end

-- Mostly copied from contents of "CompactUnitFrame_DefaultEventHandler", with some little changes to avoid taints
function manager:ProcessEvent(frame, event, ...)
	DefaultEventProc(frame, event, ...)
end

-- Register default events for the frame, this is only for saving some lines of code in case you called frame:UnregisterAllevents() and wants
-- to register back later.
function manager:RegisterDefaultEvents(frame)
	frame:RegisterEvent("PLAYER_ENTERING_WORLD")
	frame:RegisterEvent("UNIT_HEALTH")
	frame:RegisterEvent("UNIT_MAXHEALTH")
	frame:RegisterEvent("UNIT_POWER")
	frame:RegisterEvent("UNIT_MAXPOWER")
	frame:RegisterEvent("UNIT_DISPLAYPOWER")
	frame:RegisterEvent("UNIT_POWER_BAR_SHOW")
	frame:RegisterEvent("UNIT_POWER_BAR_HIDE")
	frame:RegisterEvent("UNIT_NAME_UPDATE")
	frame:RegisterEvent("UNIT_AURA")
	frame:RegisterEvent("PLAYER_TARGET_CHANGED")
	frame:RegisterEvent("UNIT_THREAT_SITUATION_UPDATE")
	frame:RegisterEvent("PLAYER_REGEN_ENABLED")
	frame:RegisterEvent("PLAYER_REGEN_DISABLED")
	frame:RegisterEvent("UNIT_CONNECTION")
	frame:RegisterEvent("UNIT_HEAL_PREDICTION")
	frame:RegisterEvent("PLAYER_ROLES_ASSIGNED")
	frame:RegisterEvent("UNIT_ENTERED_VEHICLE")
	frame:RegisterEvent("UNIT_EXITED_VEHICLE")
	frame:RegisterEvent("UNIT_PET")
	frame:RegisterEvent("READY_CHECK")
	frame:RegisterEvent("READY_CHECK_FINISHED")
	frame:RegisterEvent("READY_CHECK_CONFIRM")
	frame:RegisterEvent("PARTY_MEMBER_DISABLE")
	frame:RegisterEvent("PARTY_MEMBER_ENABLE")
end